package javax.management.remote;

import com.sun.jmx.mbeanserver.Util;
import java.util.StringTokenizer;
import sun.reflect.misc.ReflectUtil;
import java.util.ServiceLoader;
import java.util.Iterator;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.net.MalformedURLException;
import java.util.Collections;
import com.sun.jmx.remote.util.EnvHelp;
import java.util.HashMap;
import java.util.Map;
import java.io.IOException;
import com.sun.jmx.remote.util.ClassLogger;

public class JMXConnectorFactory
{
    public static final String DEFAULT_CLASS_LOADER = "jmx.remote.default.class.loader";
    public static final String PROTOCOL_PROVIDER_PACKAGES = "jmx.remote.protocol.provider.pkgs";
    public static final String PROTOCOL_PROVIDER_CLASS_LOADER = "jmx.remote.protocol.provider.class.loader";
    private static final String PROTOCOL_PROVIDER_DEFAULT_PACKAGE = "com.sun.jmx.remote.protocol";
    private static final ClassLogger logger;
    
    private JMXConnectorFactory() {
    }
    
    public static JMXConnector connect(JMXServiceURL jmxServiceURL) throws IOException {
        return connect(jmxServiceURL, null);
    }
    
    public static JMXConnector connect(JMXServiceURL jmxServiceURL, Map<String, ?> map) throws IOException {
        if (jmxServiceURL == null) {
            throw new NullPointerException("Null JMXServiceURL");
        }
        JMXConnector jmxConnector = newJMXConnector(jmxServiceURL, map);
        jmxConnector.connect(map);
        return jmxConnector;
    }
    
    private static <K, V> Map<K, V> newHashMap() {
        return new HashMap();
    }
    
    private static <K> Map<K, Object> newHashMap(Map<K, ?> m) {
        return new HashMap(m);
    }
    
    public static JMXConnector newJMXConnector(JMXServiceURL jmxServiceURL, Map<String, ?> attributes) throws IOException {
        Map<String, Object> m;
        if (attributes == null) {
            m = newHashMap();
        }
        else {
            EnvHelp.checkAttributes(attributes);
            m = newHashMap(attributes);
        }
        ClassLoader resolveClassLoader = resolveClassLoader(m);
        Class<JMXConnectorProvider> clazz = JMXConnectorProvider.class;
        String protocol = jmxServiceURL.getProtocol();
        JMXConnectorProvider jmxConnectorProvider = (JMXConnectorProvider)getProvider(jmxServiceURL, m, "ClientProvider", clazz, resolveClassLoader);
        IOException cause = null;
        if (jmxConnectorProvider == null) {
            if (resolveClassLoader != null) {
                try {
                    JMXConnector connectorAsService = getConnectorAsService(resolveClassLoader, jmxServiceURL, m);
                    if (connectorAsService != null) {
                        return connectorAsService;
                    }
                }
                catch (JMXProviderException ex) {
                    throw ex;
                }
                catch (IOException ex2) {
                    cause = ex2;
                }
            }
            jmxConnectorProvider = (JMXConnectorProvider)getProvider(protocol, "com.sun.jmx.remote.protocol", JMXConnectorFactory.class.getClassLoader(), "ClientProvider", clazz);
        }
        if (jmxConnectorProvider == null) {
            MalformedURLException throwable = new MalformedURLException("Unsupported protocol: " + protocol);
            if (cause == null) {
                throw throwable;
            }
            throw (MalformedURLException)EnvHelp.initCause(throwable, cause);
        }
        else {
            return jmxConnectorProvider.newJMXConnector(jmxServiceURL, Collections.unmodifiableMap(m));
        }
    }
    
    private static String resolvePkgs(Map<String, ?> map) throws JMXProviderException {
        Object o = null;
        if (map != null) {
            o = map.get("jmx.remote.protocol.provider.pkgs");
        }
        if (o == null) {
            o = AccessController.doPrivileged(new PrivilegedAction<String>() {
                @Override
                public String run() {
                    return System.getProperty("jmx.remote.protocol.provider.pkgs");
                }
            });
        }
        if (o == null) {
            return null;
        }
        if (!(o instanceof String)) {
            throw new JMXProviderException("Value of jmx.remote.protocol.provider.pkgs parameter is not a String: " + o.getClass().getName());
        }
        String str = (String)o;
        if (str.trim().equals("")) {
            return null;
        }
        if (str.startsWith("|") || str.endsWith("|") || str.indexOf("||") >= 0) {
            throw new JMXProviderException("Value of jmx.remote.protocol.provider.pkgs contains an empty element: " + str);
        }
        return str;
    }
    
    static <T> T getProvider(JMXServiceURL jmxServiceURL, Map<String, Object> map, String s, Class<T> clazz, ClassLoader classLoader) throws IOException {
        String protocol = jmxServiceURL.getProtocol();
        String resolvePkgs = resolvePkgs(map);
        Object provider = null;
        if (resolvePkgs != null) {
            provider = getProvider(protocol, resolvePkgs, classLoader, s, clazz);
            if (provider != null) {
                map.put("jmx.remote.protocol.provider.class.loader", (classLoader != provider.getClass().getClassLoader()) ? wrap(classLoader) : classLoader);
            }
        }
        return provider;
    }
    
    static <T> Iterator<T> getProviderIterator(Class<T> service, ClassLoader loader) {
        return ServiceLoader.load(service, loader).iterator();
    }
    
    private static ClassLoader wrap(final ClassLoader classLoader) {
        return (classLoader != null) ? ((ClassLoader)AccessController.doPrivileged(new PrivilegedAction<ClassLoader>() {
            @Override
            public ClassLoader run() {
                return new ClassLoader(classLoader) {
                    @Override
                    protected Class<?> loadClass(String s, boolean resolve) throws ClassNotFoundException {
                        ReflectUtil.checkPackageAccess(s);
                        return super.loadClass(s, resolve);
                    }
                };
            }
        })) : null;
    }
    
    private static JMXConnector getConnectorAsService(ClassLoader classLoader, JMXServiceURL obj, Map<String, ?> map) throws IOException {
        Iterator<JMXConnectorProvider> providerIterator = getProviderIterator(JMXConnectorProvider.class, classLoader);
        IOException ex = null;
        while (providerIterator.hasNext()) {
            JMXConnectorProvider jmxConnectorProvider = (JMXConnectorProvider)providerIterator.next();
            try {
                return jmxConnectorProvider.newJMXConnector(obj, map);
            }
            catch (JMXProviderException ex2) {
                throw ex2;
            }
            catch (Exception ex3) {
                if (JMXConnectorFactory.logger.traceOn()) {
                    JMXConnectorFactory.logger.trace("getConnectorAsService", "URL[" + obj + "] Service provider exception: " + ex3);
                }
                if (!(ex3 instanceof MalformedURLException) && ex == null) {
                    if ((ex3 instanceof IOException)) {
                        ex = (IOException)ex3;
                    }
                    else {
                        ex = (IOException)EnvHelp.initCause(new IOException(ex3.getMessage()), ex3);
                    }
                }
                else {
                    continue;
                }
            }
        }
        if (ex == null) {
            return null;
        }
        throw ex;
    }
    
    static <T> T getProvider(String s, String str, ClassLoader loader, String str2, Class<T> clazz) throws IOException {
        StringTokenizer stringTokenizer = new StringTokenizer(str, "|");
        while (stringTokenizer.hasMoreTokens()) {
            String string = stringTokenizer.nextToken() + "." + protocol2package(s) + "." + str2;
            Class<Object> forName;
            try {
                forName = Class.forName(string, true, loader);
            }
            catch (ClassNotFoundException ex) {
                continue;
            }
            if (!clazz.isAssignableFrom(forName)) {
                throw new JMXProviderException("Provider class does not implement " + clazz.getName() + ": " + forName.getName());
            }
            Class<Object> clazz2 = (Class)Util.cast(forName);
            try {
                return clazz2.newInstance();
            }
            catch (Exception ex2) {
                throw new JMXProviderException("Exception when instantiating provider [" + string + "]", ex2);
            }
        }
        return null;
    }
    
    static ClassLoader resolveClassLoader(Map<String, ?> map) {
        ClassLoader contextClassLoader = null;
        if (map != null) {
            try {
                contextClassLoader = (ClassLoader)map.get("jmx.remote.protocol.provider.class.loader");
            }
            catch (ClassCastException ex) {
                throw new IllegalArgumentException("The ClassLoader supplied in the environment map using the jmx.remote.protocol.provider.class.loader attribute is not an instance of java.lang.ClassLoader");
            }
        }
        if (contextClassLoader == null) {
            contextClassLoader = Thread.currentThread().getContextClassLoader();
            return contextClassLoader;
        }
        return contextClassLoader;
    }
    
    private static String protocol2package(String s) {
        return s.replace('+', '.').replace('-', '_');
    }
    
    static {
        logger = new ClassLogger("javax.management.remote.misc", "JMXConnectorFactory");
    }
}
